/**
 * ----------------------------------------------------------------
 * Copyright (C) 2012 dx0ne
 * ----------------------------------------------------------------
 *
 * File:		Enemy.as
 * Info:		
 * Package		 
 *
 * @author		dx0ne <dx0ne@laislacorporation.com>
 */
package  
{
	import starling.display.MovieClip;
	import starling.display.DisplayObjectContainer;
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.core.Starling;
	import starling.utils.deg2rad;
	
	public class Enemy extends DisplayObjectContainer
	{
		private var bodySpr:Sprite;
		private var body:Image;
		private var label:Image;
		
		public var isAlive:Boolean = false;
		
		public var isLeft:Boolean = false;
		
		protected var lifeTime:Number = 1;
		protected var currentTime:Number = 0;
		
		protected var startAlpha:Number = 0;
		protected var diffAlpha:Number = 1;
		protected var startX:Number = 640 >> 1;
		protected var startY:Number = 240+30;
		protected var diffX:Number = 0;
		protected var diffY:Number = 240-50;
		protected var startScale:Number = 0;
		protected var diffScale:Number = 5;
		
		private var offsetX:Number = 0;

		private var bdX:int = 0;
		private var bdY:int = 0;
		
		protected var timeFactor:Number=0;
		public var side:int;
		public var canExplode:Boolean = true;
		
		public function Enemy() 
		{
			bodySpr = new Sprite();
			addChild(bodySpr);
			
			body = new Image(AssetsGenerator.getTexture("enemy0.png", AssetsGenerator.defaultAtlas));
			bodySpr.addChild(body);
			
			var mc:MovieClip = new MovieClip(AssetsGenerator.defaultAtlas.getTextures("dangerbar"), 5);
			Starling.juggler.add(mc);
			mc.x = 32;
			mc.y = 7;
			bodySpr.addChild(mc);
			//mc.pause();

			mc = new MovieClip(AssetsGenerator.defaultAtlas.getTextures("dangerbar"), 5);
			Starling.juggler.add(mc);
			//mc.currentFrame = 1;
			mc.x = 63;
			mc.y = 7;
			bodySpr.addChild(mc);
			
			label = new Image(AssetsGenerator.getTexture("dangerlabel.png", AssetsGenerator.defaultAtlas));
			label.x = 46;
			label.y = 15;
			bodySpr.addChild(label);
			
			bodySpr.x = -bodySpr.width>>1;
			bodySpr.y = -bodySpr.height;
			
			visible = false;
		}
		
		public function init():void
		{
			bdX = 255 * Math.random();
			bdY = 255 * Math.random();
			currentTime = 0;
			lifeTime = 1.0;
			
			isAlive = true;
			
			startY = 240 + 25;
			startX = 320-offsetX;
			diffX = -190;
			diffY = 230;

			startAlpha = 0.1;
			
			this.x = startX;
			this.y = startY;
			this.scaleX = this.scaleY = startScale=0;
			bodySpr.scaleX = 1;
			this.visible = true;
			diffScale = 2;
			//spr.rotation = -deg2rad(offsetRot);
			//bodySpr.rotation = deg2rad(90);
			isLeft = true;
			side = -1;
			canExplode = true;
			bodySpr.rotation = 0;
		}
		
		public function leftInit():void
		{
			this.isLeft = false;
			side = 1;
			diffX *= -1;
			//bodySpr.scaleX = -1;
			//bodySpr.rotation = deg2rad(offsetRot);
			startX = 320 + offsetX;
			this.x = startX;
		}
		public function update(dt:Number):void
		{
			currentTime += dt;
			timeFactor = currentTime / lifeTime;
			if (timeFactor > 1)
			{
				timeFactor = 1;
			}
			//this.x = noEasing(currentTime, startX, diffX, lifeTime);
			//this.y = noEasing(currentTime, startY, diffY, lifeTime);
			this.x = startX + diffX*timeFactor;
			this.y = startY + diffY*timeFactor;
			//this.scaleX = this.scaleY = startScale + diffScale*timeFactor;
			//this.alpha = startAlpha + diffAlpha * timeFactor;
			this.alpha = easeOutQubic(currentTime, startAlpha, diffAlpha, lifeTime);
			this.scaleX = this.scaleY = noEasing(currentTime, startScale, diffScale, lifeTime);
			
			bdX += 200*dt;
			if (bdX > Game.bdXmax)
			{
				bdX = 0;
				bdY++;
			}
			if (bdY > Game.bdXmax)
			{
				bdY = 0;
			}
			if(canExplode)
				body.alpha = 1 - 0.7 * Game.getFrac(bdX, bdY);
			
			if (timeFactor >= 1)			
				{
					die();
				}
		}

		public function isNearPlayer():Boolean
		{
			return timeFactor > 0.5;
		}
		
		public function explode():void
		{
			if (!canExplode) return;
			canExplode = false;
			body.alpha = 0.;
		}
		
		protected function die():void
		{
			isAlive = false;
			visible = false;
		}
		
		protected function noEasing(t:Number, b:Number, c:Number, d:Number):Number {
			return c * t / d + b;
		}
		
		protected function easeInQuantic(t:Number, b:Number, c:Number, d:Number):Number {
			var ts:Number=(t/=d)*t;
			var tc:Number=ts*t;
			return b+c*(tc*ts);
		}
		
		protected function easeInQuad(t:Number, b:Number, c:Number, d:Number):Number {
			var ts:Number=(t/=d)*t;
			return b+c*(ts);
		}
		protected function easeInOutQuantic(t:Number, b:Number, c:Number, d:Number):Number {
			var ts:Number=(t/=d)*t;
			var tc:Number=ts*t;
			return b+c*(6*tc*ts + -15*ts*ts + 10*tc);
		}
		
protected function easeOutQuantic(t:Number, b:Number, c:Number, d:Number):Number {
	var ts:Number=(t/=d)*t;
	var tc:Number=ts*t;
	return b+c*(tc*ts + -5*ts*ts + 10*tc + -10*ts + 5*t);
}		
protected function easeOutQubic(t:Number, b:Number, c:Number, d:Number):Number {
	var ts:Number=(t/=d)*t;
	var tc:Number=ts*t;
	return b+c*(tc + -3*ts + 3*t);
}
	}

}